/**
 * 
 */
package com.wily.jbley.supergnf;

import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.WeakHashMap;

import bsh.Interpreter;

import com.wily.introscope.agent.AgentNotAvailableException;
import com.wily.introscope.agent.AgentShim;
import com.wily.introscope.agent.IAgent;
import com.wily.util.feedback.IModuleFeedbackChannel;

public class ScriptEnvironment
{

    public Interpreter       interpreter     = new Interpreter();

    boolean                  inUse           = false;

    public StringBuffer      stringBuffer    = new StringBuffer();

    static Map               classLoaders    = Collections.synchronizedMap(new WeakHashMap());

    static Set               environmentRefs = Collections.synchronizedSet(new HashSet());

    static ReferenceQueue    referenceQueue  = new ReferenceQueue();

    static final ThreadLocal environment     = new ThreadLocal()
                                             {
                                                 protected Object initialValue()
                                                 {
                                                     final ScriptEnvironment scriptEnvironment = new ScriptEnvironment();
                                                     return scriptEnvironment;
                                                 }
                                             };

    static
    {
        try
        {
            final IAgent agent = AgentShim.getAgent();
            final Thread resetBshInterpreterThread = new Thread(new Runnable()
            {
                public void run()
                {
                    final IModuleFeedbackChannel log = agent.IAgent_getModuleFeedback();
                    try
                    {
                        while (true)
                        {
                            Reference clearedReference = referenceQueue.remove(10000);
                            while (clearedReference != null)
                            {
                                clearedReference.clear();
                                clearedReference = referenceQueue.poll();
                            }
                            log.debug("size: " + environmentRefs.size());
                        }
                    } catch (final Exception e)
                    {
                        log.error("Unable to clear", e);
                    }
                }
            },
                                                                "Reset BSH Interpreter");
            resetBshInterpreterThread.setDaemon(true);
            resetBshInterpreterThread.start();
        } catch (final AgentNotAvailableException e)
        {
            throw new ExceptionInInitializerError(e);
        }
    }

    public ScriptEnvironment()
    {
        super();
        resetInterpreter();
    }

    synchronized protected void resetInterpreter()
    {
        this.stringBuffer = new StringBuffer();
        this.interpreter = new Interpreter();
    }

    synchronized public void reset() throws Exception
    {
        this.interpreter.unset("data");
        this.stringBuffer.setLength(0);
        this.inUse = false;
    }

    public static ScriptEnvironment getInstance()
    {
        final ScriptEnvironment scriptEnvironment = (ScriptEnvironment) environment.get();
        synchronized (scriptEnvironment)
        {
            try
            {
                if (scriptEnvironment.inUse)
                {
                    return new ScriptEnvironment();
                } else
                {
                    return scriptEnvironment;
                }
            } finally
            {
                scriptEnvironment.inUse = true;
            }
        }
    }

    static public void caching(final Class clas)
    {
        final ClassLoader classLoader = clas.getClassLoader();
        if (classLoader == null)
        {
            return;
        }
        synchronized (classLoaders)
        {
            if (!classLoaders.containsKey(classLoader))
            {
                classLoaders.put(classLoader, Boolean.TRUE);
                final WeakReference weakReference = new WeakReference(classLoader,
                                                                      referenceQueue)
                {
                    public void clear()
                    {
                        super.clear();
                        final ScriptEnvironment scriptEnvironment = (ScriptEnvironment) environment.get();
                        scriptEnvironment.resetInterpreter();
                        environmentRefs.remove(this);
                    }
                };
                environmentRefs.add(weakReference);
            }
        }
    }
}