package com.wily.jbley.supergnf.poll;

import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.wily.introscope.agent.IAgent;
import com.wily.introscope.agent.service.IAgentService;
import com.wily.util.heartbeat.ITimestampedRunnable;
import com.wily.util.properties.IndexedProperties;
import com.wily.util.task.IExecutableItem;

public class PollingService
    implements
        IAgentService
{
    public static final Set    polls                       = Collections.synchronizedSet(new HashSet());

    public static final String PollingPeriodAgentConfigKey = "introscope.agent.gnf.poller.period";

    public static final long   DefaultPollingPeriod        = 7500;

    public static final long   WarnIfPollsTakeLongerThan   = 3000;

    public static void addPoll(Poll poll)
    {
        polls.add(poll);
    }

    private static long calculatePollingPeriod(IndexedProperties config)
    {
        int answer = config.getIntProperty(PollingPeriodAgentConfigKey,
                                           (int) DefaultPollingPeriod);

        return answer;
    }

    public static void doPolls(IAgent agent)
    {
        // quick and dirty sanity check here to prevent overloads
        long startTime = System.currentTimeMillis();
        invokeAllPolls(agent);
        long totalTime = System.currentTimeMillis() - startTime;
        if (totalTime > WarnIfPollsTakeLongerThan)
        {
            agent.IAgent_getModuleFeedback()
                 .warn("GNF Poller: performing all polls took "
                       + totalTime
                       + " millseconds.  This might have an adverse affect on overhead.");
        }
    }

    private static void invokeAllPolls(IAgent agent)
    {
        Poll[] toPoll = (Poll[]) polls.toArray(new Poll[polls.size()]);
        for (int i = 0; i < toPoll.length; i++)
        {
            Poll poll = toPoll[i];
            if (poll.refInvocationObject.get() == null)
            {
                polls.remove(poll);
            } else
            {
                poll.invoke(agent);
            }
        }
    }

    public int IAgentService_getServiceVersion()
    {
        return 2;
    }

    public void IAgentService_startService(final IAgent agent, Map parameters) throws Exception
    {
        long pollingPeriod = calculatePollingPeriod(agent.IAgent_getIndexedProperties());
        agent.IAgent_getModuleFeedback().info("GNF Poller will poll every "
                                              + pollingPeriod
                                              + " milliseconds");
        agent.IAgent_getCommonHeartbeat()
             .addBehavior(new ITimestampedRunnable()
             {
                 public void ITimestampedRunnable_execute(long nowInMillis)
                 {
                     // asynch jump here so we don't tie up the main heartbeat
                     // thread
                     agent.IAgent_getSharedAsyncQueue()
                          .IExecutionQueue_addExecutableItem(new IExecutableItem()
                          {
                              public void IExecutableItem_execute()
                              {
                                  doPolls(agent);
                              }

                              public void IExecutableItem_executionAborted()
                              {
                                  // nop
                              }

                              public boolean IExecutableItem_shouldStillExecute()
                              {
                                  return true;
                              }

                          });
                 }
             },
                          "GNF Poller",
                          true,
                          pollingPeriod,
                          false);
    }

}
