package com.wily.jbley.supergnftests;

import junit.framework.TestCase;

import com.wily.introscope.agent.AgentShim;
import com.wily.introscope.agent.IAgent;
import com.wily.introscope.agent.trace.ITracerFactory;
import com.wily.introscope.agent.trace.InvocationData;
import com.wily.introscope.agent.trace.ProbeIdentification;
import com.wily.introscope.agent.trace.ProbeInformation;
import com.wily.jbley.supergnf.SuperGNFErrorReportingTracer;
import com.wily.util.properties.AttributeListing;

public class ErrorReporterTests
    extends TestCase
{
    protected IAgent agent;

    private void gc()
    {
        for (int i = 0; i < 3; i++)
        {
            System.gc();
            System.runFinalization();
            try
            {
                Thread.sleep(50);
            } catch (Exception e)
            {
                // nop
            }
        }
    }

    protected void setUp() throws Exception
    {
        this.agent = AgentShim.getAgent();
        this.agent.debug_setRetrievingValues(true);
    }

    protected void tearDown() throws Exception
    {
        this.agent.debug_setRetrievingValues(false);
        gc();
        Thread.sleep(1000);
    }

    public void testBadScript() throws Exception
    {
        String o = new String("hello world");
        ProbeIdentification pi = new ProbeIdentification("probe/someClass",
                                                         "someMethod",
                                                         "()V",
                                                         "probe/SomeClass");

        AttributeListing props = new AttributeListing();
        props.put(SuperGNFErrorReportingTracer.ErrorConditionKey,
                  "data.getInvocationObject().length() > 2");
        props.put(SuperGNFErrorReportingTracer.ErrorMessageKey,
                  "ksjdkfjksjdg8hnk;02ujhn8s&&^%%$%");
        props.put(ITracerFactory.kNameKey, "Hello World:Errors Per Interval");

        SuperGNFErrorReportingTracer t = new SuperGNFErrorReportingTracer(this.agent,
                                                                          props,
                                                                          pi,
                                                                          o);
        InvocationData data = InvocationData.debug_createInvocationData(this.agent,
                                                                        new ProbeInformation(this.agent,
                                                                                             pi,
                                                                                             new ITracerFactory[0]),
                                                                        o);
        t.ITracer_startTrace(0, data);
        t.ITracer_finishTrace(0, data);

    }

    public void testMissingParam() throws Exception
    {
        String o = new String("hello world");
        ProbeIdentification pi = new ProbeIdentification("probe/someClass",
                                                         "someMethod",
                                                         "()V",
                                                         "probe/SomeClass");

        AttributeListing props = new AttributeListing();
        // props.put(SuperGNFErrorReportingTracer.ErrorConditionKey,
        // "data.getInvocationObject().length() > 2");
        // props.put(SuperGNFErrorReportingTracer.ErrorMessageKey,
        // "data.getInvocationObject()");
        props.put(ITracerFactory.kNameKey, "Hello World:Errors Per Interval");

        SuperGNFErrorReportingTracer t = new SuperGNFErrorReportingTracer(this.agent,
                                                                          props,
                                                                          pi,
                                                                          o);
        InvocationData data = InvocationData.debug_createInvocationData(this.agent,
                                                                        new ProbeInformation(this.agent,
                                                                                             pi,
                                                                                             new ITracerFactory[0]),
                                                                        o);
        t.ITracer_startTrace(0, data);
        t.ITracer_finishTrace(0, data);
    }

    public void testPollingTracer() throws Exception
    {
        String o = new String("hello world");
        ProbeIdentification pi = new ProbeIdentification("probe/someClass",
                                                         "someMethod",
                                                         "()V",
                                                         "probe/SomeClass");

        AttributeListing props = new AttributeListing();
        props.put(SuperGNFErrorReportingTracer.ErrorConditionKey,
                  "data.getInvocationObject().length() > 2");
        props.put(SuperGNFErrorReportingTracer.ErrorMessageKey,
                  "data.getInvocationObject()");
        props.put(ITracerFactory.kNameKey, "Hello World:Errors Per Interval");

        SuperGNFErrorReportingTracer t = new SuperGNFErrorReportingTracer(this.agent,
                                                                          props,
                                                                          pi,
                                                                          o);
        InvocationData data = InvocationData.debug_createInvocationData(this.agent,
                                                                        new ProbeInformation(this.agent,
                                                                                             pi,
                                                                                             new ITracerFactory[0]),
                                                                        o);
        t.ITracer_startTrace(0, data);
        t.ITracer_finishTrace(0, data);

        assertEquals(o, t.createErrorMessage(data));
        assertTrue(t.hasError(data));
    }
}
