package com.wily.jbley.supergnftests;

import java.util.HashMap;

import junit.framework.TestCase;

import com.wily.introscope.agent.AgentShim;
import com.wily.introscope.agent.IAgent;
import com.wily.introscope.agent.trace.ITracerFactory;
import com.wily.introscope.agent.trace.InvocationData;
import com.wily.introscope.agent.trace.ProbeIdentification;
import com.wily.introscope.agent.trace.ProbeInformation;
import com.wily.introscope.spec.metric.AgentMetric;
import com.wily.introscope.spec.metric.MetricTypes;
import com.wily.jbley.supergnf.SuperGNF;

public class SuperGnfTests
    extends TestCase
{

    private IAgent           agent;

    private ProbeInformation probeInfo;

    protected void setUp() throws Exception
    {
        super.setUp();
        this.agent = AgentShim.getAgent();
        this.probeInfo = new ProbeInformation(this.agent,
                                              new ProbeIdentification("probe.ClassName",
                                                                      "probeMethod",
                                                                      "()V",
                                                                      "runtime.ClassName"),
                                              new ITracerFactory[0]);
    }

    public SuperGnfTests(final String test)
    {
        super(test);
    }

    public SuperGnfTests()
    {
        super();
    }

    protected void tearDown() throws Exception
    {
        super.tearDown();
    }

    public void testBasic() throws Exception
    {
        final Object thisObj = new HashMap();
        final InvocationData data = InvocationData.debug_createInvocationData(this.agent,
                                                                              this.probeInfo,
                                                                              thisObj);
        final SuperGNF gnf = new SuperGNF();

        final String[][] tests = new String[][] {
                { "nothing changes", "nothing changes" },
                {
                        "Foo:{{data.getClass().getName()}}",
                        "Foo:" + InvocationData.class.getName() },
                {
                        "Foo:{{data.getInvocationObject().getClass().getInterfaces()[0]}}",
                        "Foo:" + thisObj.getClass().getInterfaces()[0] },
                {
                        "Foo:{{x=data.getInvocationObject().getClass().getInterfaces();java.util.Arrays.asList(x).get(0)}}",
                        "Foo:" + thisObj.getClass().getInterfaces()[0] },
                { "Foo|{{4+1}}|{{2+2}}:Stuff", "Foo|5|4:Stuff" }, };
        for (int i = 0; i < tests.length; i++)
        {
            final String in = tests[i][0];
            final String expected = tests[i][1];
            final String result = gnf.INameFormatter_format(data, in);
            System.out.println("Test returned " + result);
            assertEquals("For input ["
                         + in
                         + "] expected ["
                         + expected
                         + "] but got ["
                         + result
                         + "]", expected, result);
        }
    }

    public void testCompilationError() throws Exception
    {
        final Object thisObj = new HashMap();
        final InvocationData data = InvocationData.debug_createInvocationData(this.agent,
                                                                              this.probeInfo,
                                                                              thisObj);
        final SuperGNF gnf = new SuperGNF();

        gnf.INameFormatter_format("{{skdj;-981hn0mnsh,}}", data);
        assertFalse(gnf.INameFormatter_canUseRuntimeFullClassNameCaching());
    }

    public void testMetricNameSafety() throws Exception
    {
        final String[][] tests = new String[][] {
                { "nothing changes", "nothing changes" },
                { " string is trimmed ", " string is trimmed" },
                { "| and : get replaced", "_ and _ get replaced" },
                { "", "<empty>" }, };
        for (int i = 0; i < tests.length; i++)
        {
            final String in = tests[i][0];
            final String expected = tests[i][1];
            final String result = SuperGNF.makeSafeForMetrics(in);
            assertEquals("For input ["
                         + in
                         + "] expected ["
                         + expected
                         + "] but got ["
                         + result
                         + "]", expected, result);
            // None of these should throw
            AgentMetric.getAgentMetric(result, MetricTypes.kIntegerDuration);
            AgentMetric.getAgentMetric("Prefix:" + result,
                                       MetricTypes.kIntegerDuration);
            AgentMetric.getAgentMetric("Prefix|MoreStuff:" + result,
                                       MetricTypes.kIntegerDuration);
        }
    }

    public void testSlow() throws Exception
    {
        final Object thisObj = new HashMap();
        final InvocationData data = InvocationData.debug_createInvocationData(this.agent,
                                                                              this.probeInfo,
                                                                              thisObj);
        final SuperGNF gnf = new SuperGNF();

        gnf.INameFormatter_format("{{Thread.sleep(1100)}}", data);
        Thread.sleep(1000);
    }

}
